@@ -3,6 +3,18 @@ import music21 as m21
 import time
 import requests
 from pathlib import Path
+import xml.etree.ElementTree as ET
+
+# MEINSURI = 'http://www.music-encoding.org/ns/mei'
+# MEINS = '{%s}' % MEINSURI
+# mei_doc = ET.fromstring(requests.get(path).text)
+#   # Find the title from the MEI file and update the Music21 Score metadata
+# title = mei_doc.find(f'{MEINS}meiHead//{MEINS}titleStmt/{MEINS}title').text
+# score.metadata.title = title
+# mei_doc = ET.fromstring(requests.get(path).text)
+#   # Find the composer from the MEI file and update the Music21 Score metadata
+# composer = mei_doc.find(f'{MEINS}meiHead//{MEINS}respStmt/{MEINS}persName').text
+# score.metadata.composer = composer
 
 # An extension of the music21 note class with more information easily accessible
 class NoteListElement:
@@ -27,9 +39,12 @@ class NoteListElement:
         note duration
     piece_url : str
         piece url for note
+    prev_note : NoteListElement
+        prior non-rest note element
     """
-    def __init__(self, note: m21.note.Note, metadata, part, partNumber, duration, piece_url):
+    def __init__(self, note: m21.note.Note, metadata, part, partNumber, duration, piece_url, prev_note=None):
         self.note = note
+        self.prev_note = prev_note
         self.offset = self.note.offset
         self.id = self.note.id
         self.metadata = metadata
@@ -100,6 +115,7 @@ class CorpusBase:
         """
         pure_notes = []
         urls_index = 0
+        prev_note = None
         for score in self.scores:
             parts = score.getElementsByClass(stream.Part)
             for part in parts:
@@ -107,14 +123,17 @@ class CorpusBase:
                 for note in noteList:
                     if note.tie is not None:
                         if note.tie.type == 'start':
-                            note_obj = NoteListElement(note, score.metadata, part.partName, score.index(part), note.quarterLength, self.paths[urls_index])
+                            note_obj = NoteListElement(note, score.metadata, part.partName, score.index(part), note.quarterLength, self.paths[urls_index], prev_note)
                             pure_notes.append(note_obj)
                         else:
                             pure_notes[len(pure_notes)-1].duration += note.quarterLength
                     else:
-                        note_obj = NoteListElement(note, score.metadata, part.partName, score.index(part), note.quarterLength, self.paths[urls_index])
+                        note_obj = NoteListElement(note, score.metadata, part.partName, score.index(part), note.quarterLength, self.paths[urls_index], prev_note)
                         pure_notes.append(note_obj)
-                note_obj = NoteListElement(m21.note.Rest(), score.metadata, part.partName, score.index(part), 4.0, self.paths[urls_index])
+                    # Rests carry the last non-rest note as their prev_note
+                    if not pure_notes[-1].note.isRest:
+                        prev_note = pure_notes[-1]
+                note_obj = NoteListElement(m21.note.Rest(), score.metadata, part.partName, score.index(part), 4.0, self.paths[urls_index], prev_note)
                 pure_notes.append(note_obj)
             urls_index += 1
         return pure_notes
@@ -127,6 +146,7 @@ class CorpusBase:
         """
         pure_notes = []
         urls_index = 0
+        prev_note = None
         for score in self.scores:
             parts = score.getElementsByClass(stream.Part)
             for part in parts:
@@ -136,13 +156,15 @@ class CorpusBase:
                     if not note.isRest and note.nameWithOctave == prev_pitch:
                         pure_notes[len(pure_notes)-1].duration += note.quarterLength
                     else:
-                        note_obj = NoteListElement(note, score.metadata, part.partName, score.index(part), note.quarterLength, self.paths[urls_index])
+                        note_obj = NoteListElement(note, score.metadata, part.partName, score.index(part), note.quarterLength, self.paths[urls_index], prev_note)
                         pure_notes.append(note_obj)
                     if not note.isRest:
                         prev_pitch = note.nameWithOctave
                     else:
                         prev_pitch == 'Rest'
-                note_obj = NoteListElement(m21.note.Rest(), score.metadata, part.partName, score.index(part), 4.0, self.paths[urls_index])
+                    if not pure_notes[-1].note.isRest:
+                        prev_note = pure_notes[-1]
+                note_obj = NoteListElement(m21.note.Rest(), score.metadata, part.partName, score.index(part), 4.0, self.paths[urls_index], prev_note)
                 pure_notes.append(note_obj)
             urls_index += 1
         return pure_notes
@@ -158,6 +180,7 @@ class CorpusBase:
         """
         pure_notes = []
         urls_index = 0
+        prev_note = None
         for score in self.scores:
             parts = score.getElementsByClass(stream.Part)
             for part in parts:
@@ -169,8 +192,10 @@ class CorpusBase:
                             for point in offsets:
                                 #print(note.offset, point)
                                 if point >= note.offset and point < (note.offset + note.quarterLength):
-                                    note_obj = NoteListElement(note, score.metadata, part.partName, score.index(part), note.quarterLength, self.paths[urls_index])
+                                    note_obj = NoteListElement(note, score.metadata, part.partName, score.index(part), note.quarterLength, self.paths[urls_index], prev_note)
                                     pure_notes.append(note_obj)
+                                    if not pure_notes[-1].note.isRest:
+                                        prev_note = pure_notes[-1]
                                     break
             urls_index += 1
         return pure_notes
@@ -186,6 +211,7 @@ class CorpusBase:
         """
         pure_notes = []
         urls_index = 0
+        prev_note = None
         for score in self.scores:
             for part in score.getElementsByClass(stream.Part):
                 counter = 0
@@ -197,15 +223,17 @@ class CorpusBase:
                             note_at_offset = item
                             break
                     if note_at_offset:
-                        note_obj = NoteListElement(note_at_offset, score.metadata, part.partName, score.index(part), min_offset, self.paths[urls_index])
+                        note_obj = NoteListElement(note_at_offset, score.metadata, part.partName, score.index(part), min_offset, self.paths[urls_index], prev_note)
                         note_obj.offset = counter
                         pure_notes.append(note_obj)
                     else:
-                        note_obj = NoteListElement(m21.note.Rest(), score.metadata, part.partName, score.index(part), min_offset, self.paths[urls_index])
+                        note_obj = NoteListElement(m21.note.Rest(), score.metadata, part.partName, score.index(part), min_offset, self.paths[urls_index], prev_note)
                         note_obj.offset = counter
                         pure_notes.append(note_obj)
                     counter += min_offset
-            note_obj = NoteListElement(m21.note.Rest(), score.metadata, part.partName, score.index(part), 4.0, self.paths[urls_index])
+                    if not pure_notes[-1].note.isRest:
+                        prev_note = pure_notes[-1]
+            note_obj = NoteListElement(m21.note.Rest(), score.metadata, part.partName, score.index(part), 4.0, self.paths[urls_index], prev_note)
         urls_index += 1
         return pure_notes
 
@@ -256,19 +284,22 @@ class ScoreBase:
         """
         pure_notes = []
         parts = self.score.getElementsByClass(stream.Part)
+        prev_note = None
         for part in parts:
             noteList = part.flat.getElementsByClass(['Note', 'Rest'])
             for note in noteList:
                 if note.tie is not None:
                     if note.tie.type == 'start':
-                        note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url)
+                        note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url, prev_note)
                         pure_notes.append(note_obj)
                     else:
                         pure_notes[len(pure_notes)-1].duration += note.quarterLength
                 else:
-                    note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url)
+                    note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url, prev_note)
                     pure_notes.append(note_obj)
-            note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), 4.0, self.url)
+                if not pure_notes[-1].note.isRest:
+                    prev_note = pure_notes[-1]
+            note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), 4.0, self.url, prev_note)
             pure_notes.append(note_obj)
         return pure_notes
 
@@ -281,6 +312,7 @@ class ScoreBase:
         """
         pure_notes = []
         urls_index = 0
+        prev_note = None
         parts = self.score.getElementsByClass(stream.Part)
         for part in parts:
             noteList = part.flat.getElementsByClass(['Note', 'Rest'])
@@ -289,13 +321,15 @@ class ScoreBase:
                 if not note.isRest and note.nameWithOctave == prev_pitch:
                     pure_notes[len(pure_notes)-1].duration += note.quarterLength
                 else:
-                    note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url)
+                    note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url, prev_note)
                     pure_notes.append(note_obj)
                 if not note.isRest:
                     prev_pitch = note.nameWithOctave
                 else:
                     prev_pitch == 'Rest'
-            note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), 4.0, self.url)
+                if not pure_notes[-1].note.isRest:
+                    prev_note = pure_notes[-1]
+            note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), 4.0, self.url, prev_note)
             pure_notes.append(note_obj)
         urls_index += 1
         return pure_notes
@@ -312,13 +346,16 @@ class ScoreBase:
         """
         pure_notes = []
         parts = self.score.getElementsByClass(stream.Part)
+        prev_note = None
         for part in parts:
             noteList = part.flat.getElementsByClass(['Note', 'Rest'])
             for note in noteList:
                 if note.beat in beats:
-                        note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url)
+                        note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url, prev_note)
                         pure_notes.append(note_obj)
-            note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), 4.0, self.url)
+                        if not pure_notes[-1].note.isRest:
+                            prev_note = pure_notes[-1]
+            note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), 4.0, self.url, prev_note)
             pure_notes.append(note_obj)
         return pure_notes
 
@@ -333,6 +370,7 @@ class ScoreBase:
         """
         pure_notes = []
         part_number = 0
+        prev_note = None
         parts = self.score.getElementsByClass(stream.Part)
         for part in parts:
             part_number += 1
@@ -343,8 +381,10 @@ class ScoreBase:
                     for note in voice:
                         for point in offsets:
                             if point >= note.offset and point < (note.offset + note.quarterLength):
-                                note_obj = NoteListElement(note, self.score.metadata, part.partName, part_number, note.quarterLength, self.url)
+                                note_obj = NoteListElement(note, self.score.metadata, part.partName, part_number, note.quarterLength, self.url, prev_note)
                                 pure_notes.append(note_obj)
+                                if not pure_notes[-1].note.isRest:
+                                    prev_note = pure_notes[-1]
                                 break
         return pure_notes
 
@@ -366,6 +406,7 @@ class ScoreBase:
         part_selected = self.score.getElementsByClass(stream.Part)[part]
         measures = part_selected.getElementsByClass(stream.Measure)
         measures_selected = []
+        prev_note = None
         for i in range(num_measures):
             measures_selected.append(measures[i+measure_start])
         for measure in measures_selected:
@@ -375,13 +416,15 @@ class ScoreBase:
                     print(note.offset)
                     if note.tie is not None:
                         if note.tie.type == 'start':
-                            note_obj = NoteListElement(note, self.score.metadata, part_selected.partName, part, note.quarterLength, self.url)
+                            note_obj = NoteListElement(note, self.score.metadata, part_selected.partName, part, note.quarterLength, self.url, prev_note)
                             pure_notes.append(note_obj)
                         else:
                             pure_notes[len(pure_notes)-1].duration += note.quarterLength
                     else:
-                        note_obj = NoteListElement(note, self.score.metadata, part_selected.partName, part, note.quarterLength, self.url)
+                        note_obj = NoteListElement(note, self.score.metadata, part_selected.partName, part, note.quarterLength, self.url, prev_note)
                         pure_notes.append(note_obj)
+                    if not pure_notes[-1].note.isRest:
+                        prev_note = pure_notes[-1]
         return pure_notes
 
     # Allows for specific selection in terms of measures, but gets all parts/instruments
@@ -397,6 +440,7 @@ class ScoreBase:
             measures until end measure
         """
         pure_notes = []
+        prev_note = None
         parts = self.score.getElementsByClass(stream.Part)
         for part in parts:
             measures = part.getElementsByClass(stream.Measure)
@@ -409,15 +453,17 @@ class ScoreBase:
                     for note in voice:
                         if note.tie is not None:
                             if note.tie.type == 'start':
-                                note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url)
+                                note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url, prev_note)
                                 pure_notes.append(note_obj)
                             else:
                                 pure_notes[len(pure_notes)-1].duration += note.quarterLength
                         else:
-                            note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url)
+                            note_obj = NoteListElement(note, self.score.metadata, part.partName, self.score.index(part), note.quarterLength, self.url, prev_note)
                             pure_notes.append(note_obj)
+                        if not pure_notes[-1].note.isRest:
+                            prev_note = pure_notes[-1]
             # Added rest to ensure parts don't overlap
-            note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), 4.0, self.url)
+            note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), 4.0, self.url, prev_note)
             pure_notes.append(note_obj)
         return pure_notes
 
@@ -431,6 +477,7 @@ class ScoreBase:
             sample every x offset- 2 will sample every half note, 1 every quarter note, etc.
         """
         pure_notes = []
+        prev_note = None
         for part in self.score.getElementsByClass(stream.Part):
             counter = 0
             while counter < self.score.highestTime - min_offset:
@@ -441,15 +488,17 @@ class ScoreBase:
                         note_at_offset = item
                         break
                 if note_at_offset:
-                    note_obj = NoteListElement(note_at_offset, self.score.metadata, part.partName, self.score.index(part), min_offset, self.url)
+                    note_obj = NoteListElement(note_at_offset, self.score.metadata, part.partName, self.score.index(part), min_offset, self.url, prev_note)
                     note_obj.offset = counter
                     pure_notes.append(note_obj)
                 else:
-                    note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), min_offset, self.url)
+                    note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), min_offset, self.url, prev_note)
                     note_obj.offset = counter
                     pure_notes.append(note_obj)
+                if not pure_notes[-1].note.isRest:
+                    prev_note = pure_notes[-1]
                 counter += min_offset
-        note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), 4.0, self.url)
+        note_obj = NoteListElement(m21.note.Rest(), self.score.metadata, part.partName, self.score.index(part), 4.0, self.url, prev_note)
         return pure_notes
 
 class VectorInterval:
@@ -615,6 +664,55 @@ class PatternMatches:
             " and ending in measure " + str(match.last_note.note.measureNumber) + ". Notes lengths: " + str(match.durations))
         print("Said pattern or similar appeared " + str(len(self.matches)) + " times.\n")
 
+class ClassifiedMatch:
+    """
+    Group of matches classified to be a periodic entry, imitative duo, or fuga
+
+    Attributes
+    ----------
+    matches : list
+        list of Match objects found to be matching the pattern
+    type : str
+        either "periodic entry", "imitative duo", or "fuga" depending on match classification
+    pattern : list
+        interval pattern that the matches have in common
+    ema : str
+        ema address for the series of patterns
+    ema_url : str
+        url to download mei slice for the series of patterns
+    """
+    def __init__(self, matches: list, type):
+        """
+        Parameters
+        ----------
+        matches : list
+            list of Match objects found to be matching the pattern
+        type : str
+            either "periodic entry", "imitative duo", or "fuga" depending on match classification
+        """
+        self.matches = matches
+        self.type = type
+        self.pattern = self.matches[0].pattern
+
+        ema_measures = ""
+        ema_parts = ""
+        ema_beats = ""
+        for match in self.matches:
+            ema_measures += str(match.first_note.note.measureNumber) + "-" + str(match.last_note.note.measureNumber) + ","
+            for i in range(match.last_note.note.measureNumber - match.first_note.note.measureNumber + 1):
+                ema_parts += str(match.first_note.partNumber) + ","
+            ema_beats += "@" + str(match.first_note.note.beat) + "-end,"
+            for j in range(match.last_note.note.measureNumber - match.first_note.note.measureNumber - 1):
+                ema_beats += "@start-end,"
+            ema_beats += "@start-" + str(match.last_note.note.beat) + ","
+        self.ema = ema_measures[0:len(ema_measures)-1] + "/" + ema_parts[0:len(ema_parts)-1] + "/" + ema_beats[0:len(ema_beats)-1]
+
+        try:
+            splice = self.matches[0].first_note.piece_url.index('mei/')
+            self.ema_url = "https://ema.crimproject.org/https%3A%2F%2Fcrimproject.org%2Fmei%2F" + str(self.matches[0].first_note.piece_url[splice + 4:]) + "/" + str(self.ema)
+        except:
+            self.ema_url = "File must be a crim url (not a file path) to have a valid EMA url"
+
 # Potential redesign needed due to unstable nature of having user give over patterns_data
 # Potential fix is reincorporating into_patterns back into this method
 def find_exact_matches(patterns_data, min_matches=5):
@@ -689,11 +787,11 @@ def find_close_matches(patterns_data, min_matches, threshold):
         matches_list = PatternMatches(p, [])
         # If a pattern occurs more than the designated threshold
         for a in patterns_data:
-            match = 0
-            # Calculate the "difference" by comparing each vector with the matching on in the other pattern
+            rhytmic_match = 0
+            # Calculate the "difference" by comparing each vector with the matching one in the other pattern
             for v in range(len(a[0])):
-                match += abs(p[v] - a[0][v])
-            if match <= threshold:
+                rhytmic_match += abs(p[v] - a[0][v])
+            if rhytmic_match <= threshold:
                 close_match = Match(a[0], a[1], a[2], a[3])
                 matches_list.matches.append(close_match)
         if len(matches_list.matches) > min_matches:
@@ -878,10 +976,20 @@ def export_to_csv(matches: list):
     import csv
     with open(csv_name, mode='w') as matches_file:
         matches_writer = csv.writer(matches_file, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
-        matches_writer.writerow(['Pattern Generating Match', 'Pattern matched', 'Piece Title', 'Part', 'First Note Measure Number', 'Last Note Measure Number', 'Note Durations', 'EMA', 'EMA url'])
-        for match_series in matches:
-            for match in match_series.matches:
-                matches_writer.writerow([match_series.pattern, match.pattern, match.first_note.metadata.title, match.first_note.part, match.first_note.note.measureNumber, match.last_note.note.measureNumber, match.durations, match.ema, match.ema_url])
+        if type(matches[0]) == PatternMatches:
+            matches_writer.writerow(['Pattern Generating Match', 'Pattern matched', 'Piece Title', 'Part', 'First Note Measure Number', 'Last Note Measure Number', 'Note Durations', 'EMA', 'EMA url'])
+            for match_series in matches:
+                for match in match_series.matches:
+                    matches_writer.writerow([match_series.pattern, match.pattern, match.first_note.metadata.title, match.first_note.part, match.first_note.note.measureNumber, match.last_note.note.measureNumber, match.durations, match.ema, match.ema_url])
+        else:
+            matches_writer.writerow(['Pattern Generating Match', 'Classification Type', 'EMA', 'EMA url', 'Soggetti 1 Part', 'Soggetti 1 Measure', 'Soggetti 2 Part', 'Soggetti 2 Measure', 'Soggetti 3 Part', 'Soggetti 3 Measure', 'Soggetti 4 Part', 'Soggetti 4 Measure'])
+            for classified_matches in matches:
+                row_array = [classified_matches.pattern, classified_matches.type, classified_matches.ema, classified_matches.ema_url]
+                for soggetti in classified_matches.matches:
+                    row_array.append(soggetti.first_note.part)
+                    row_array.append(soggetti.first_note.note.measureNumber)
+                matches_writer.writerow(row_array)
+
     print("CSV created in your current working directory.")
 
 # For more naive usage- allows for user interaction, has return value of list of matches
@@ -933,7 +1041,7 @@ def assisted_interface():
     return matches
 
 def compare_durations(durations1, durations2, threshold):
-    """Used in classify_matches
+    """Helper for classify_matches
 
     works similarly to find_close_matches in terms of its comparison technique
     """
@@ -978,7 +1086,7 @@ def classify_matches(exact_matches: list, durations_threshold = 2):
             list of fuga, which are lists of Match objects
     """
     # TO-DO: add in factoring for durations, narrow 80 beats window
-    periodic_entries, im_duos, fuga = [], [], []
+    classified_matches = []
     for list_matches in exact_matches:
         offset_difs, offset_difs_info = [], []
         match_instance = list_matches.matches
@@ -994,47 +1102,38 @@ def classify_matches(exact_matches: list, durations_threshold = 2):
             elif offset_difs[i] == offset_difs[i + 1] and offset_difs[i] == offset_difs[i + 2]:
                 grouping = (offset_difs_info[i][0], offset_difs_info[i][1], offset_difs_info[i + 1][0], offset_difs_info[i + 1][1], offset_difs_info[i + 2][0], offset_difs_info[i + 2][1])
                 grouping = list(dict.fromkeys(grouping))
-                periodic_entries.append(grouping)
+                classified_obj = ClassifiedMatch(grouping, "periodic_entry")
+                classified_matches.append(classified_obj)
             elif offset_difs[i] == offset_difs[i + 1]:
                 grouping = (offset_difs_info[i][0], offset_difs_info[i][1], offset_difs_info[i + 1][0], offset_difs_info[i + 1][1])
                 grouping = list(dict.fromkeys(grouping))
-                periodic_entries.append(grouping)
+                classified_obj = ClassifiedMatch(grouping, "periodic entry")
+                classified_matches.append(classified_obj)
             elif offset_difs[i] == offset_difs[i + 2]:
                 grouping = (offset_difs_info[i][0], offset_difs_info[i][1], offset_difs_info[i + 2][0], offset_difs_info[i + 2][1])
                 grouping = list(dict.fromkeys(grouping))
-                im_duos.append(grouping)
+                classified_obj = ClassifiedMatch(grouping, "imitative duo")
+                classified_matches.append(classified_obj)
             else:
                 grouping = (offset_difs_info[i][0], offset_difs_info[i][1], offset_difs_info[i + 1][0], offset_difs_info[i + 1][1])
                 grouping = list(dict.fromkeys(grouping))
-                fuga.append(grouping)
+                classified_obj = ClassifiedMatch(grouping, "fuga")
+                classified_matches.append(classified_obj)
             i += 1
 
-    for entry in periodic_entries:
-        print("Periodic Entry:")
-        a = "Pattern: " + str(entry[0].pattern) + ", Locations in entry: "
-        for b in entry:
-            a += "\n- Measure " + str(b.first_note.note.measureNumber) + " in voice " + str(b.first_note.partNumber)
-        print(a)
-    for duo in im_duos:
-        print("Imitative Duo:")
-        a = "Pattern: " + str(duo[0].pattern) + ", Locations in entry: "
-        for c in duo:
-            a += "\n- Measure " + str(c.first_note.note.measureNumber) + " in voice " + str(c.first_note.partNumber)
-        print(a)
-    for f in fuga:
-        print("Fuga:")
-        a = "Pattern: " + str(f[0].pattern) + ", Locations in entry: "
-        for d in f:
-            a += "\n- Measure " + str(d.first_note.note.measureNumber) + " in voice " + str(d.first_note.partNumber)
-        print(a)
-    classified_tuple = (periodic_entries, im_duos, fuga)
-    return classified_tuple
+    for entry in classified_matches:
+        print(str(entry.type) + ":")
+        desc_str = "Pattern: " + str(entry.pattern) + ", Locations in entry: "
+        for soggetti in entry.matches:
+            desc_str += "\n- Measure " + str(soggetti.first_note.note.measureNumber) + " in voice " + str(soggetti.first_note.partNumber)
+        print(desc_str)
 
+    return classified_matches
 
-def export_pandas(exact_matches):
+def export_pandas(matches):
     import pandas as pd
     match_data = []
-    for match_series in exact_matches:
+    for match_series in matches:
         for match in match_series.matches:
             match_dict = {
               "pattern_generating_match": match_series.pattern,
